"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
//NOTE work not completed, custom implementation of serve
var fs = require("fs-extra");
var webpack = require("webpack");
var _ = require("lodash");
var chokidar = require("chokidar");
var FriendlyErrorsWebpackPlugin = require("friendly-errors-webpack-plugin");
var spawn = require("cross-spawn");
var os = require("os");
var nodeRunner_1 = require("../helpers/nodeRunner");
var webpackConfigLoader_1 = require("../config/webpackConfigLoader");
var webpackHelper_1 = require("../helpers/webpackHelper");
var pathHelper_1 = require("./../helpers/pathHelper");
var utils_1 = require("./../helpers/utils");
var config_1 = require("../config/config");
var envHelper_1 = require("../helpers/envHelper");
var nodemon = require('nodemon');
function initEnvVars() {
    if (!process.env.APP_DIR) {
        process.env.APP_DIR = fs.realpathSync(process.cwd());
    }
    process.env.NODE_ENV = 'development';
}
function dev() {
    if (envHelper_1.default.isTsServerLang()) {
        devTs();
    }
    else {
        devJs();
    }
}
var output = [];
var scriptRunner = null;
function devTs() {
    //TODO duplication
    utils_1.default.clearConsole();
    utils_1.default.log('Starting new compilation...');
    var ts = spawn('tsc', ['--watch'], {
        cwd: pathHelper_1.default.serverRelative('./')
    });
    var compile = _.debounce(function () {
        tsCompileRequest();
    }, 200, {});
    ts.stdout.on('data', function (data) {
        var entry = _.trim(data.toString());
        var parts = entry.split(os.EOL);
        for (var _i = 0, parts_1 = parts; _i < parts_1.length; _i++) {
            var part = parts_1[_i];
            output.push(_.trim(part));
        }
        compile();
    });
    ts.stderr.on('data', function (data) {
        console.log("stderr: " + data);
    });
    ts.on('close', function (code) {
        console.log("child process exited with code " + code);
    });
    var entry = envHelper_1.default.getTsBuildEntry();
    scriptRunner = nodeRunner_1.default.init(entry);
}
function tsCompileRequest() {
    var lastMessage = output[output.length - 1];
    if (_.endsWith(lastMessage, 'File change detected. Starting incremental compilation...')) {
        scriptRunner.stop();
        utils_1.default.clearConsole();
        utils_1.default.log('Starting new compilation...');
        return;
    }
    if (_.endsWith(lastMessage, 'Compilation complete. Watching for file changes.')) {
        var errors = [];
        for (var i = output.length - 2; i >= 0; i--) {
            if (!_.endsWith(output[i], 'File change detected. Starting incremental compilation...')) {
                errors.push(output[i]);
            }
            else {
                break;
            }
        }
        return onTsCompileComplete(errors);
    }
}
function onTsCompileComplete(errors) {
    _.delay(function () {
        if (errors.length === 0) {
            utils_1.default.log('Compiled successfully!', 'green');
            scriptRunner.start();
        }
        else {
            utils_1.default.log('Failed to compile.', 'red');
            for (var _i = 0, errors_1 = errors; _i < errors_1.length; _i++) {
                var error = errors_1[_i];
                utils_1.default.log(error);
            }
        }
    }, 0);
}
function devJs() {
    var watchPath = pathHelper_1.default.serverRelative(config_1.default.paths.server.src);
    var watcher = chokidar.watch([watchPath]);
    var webpackConfig = webpackConfigLoader_1.default.loadWebpackConfig('js_dev');
    webpackConfig.plugins.push[new FriendlyErrorsWebpackPlugin()];
    var compiler = webpack(webpackConfig);
    var compile = function (cb) {
        var start = new Date();
        compiler.run(function (err, stats) {
            webpackHelper_1.default.handleErrors(err, stats, false);
            var end = new Date();
            var timeStr = utils_1.default.getFormattedTimeInterval(start, end);
            utils_1.default.clearConsole();
            utils_1.default.log("Compiled: " + timeStr, 'green');
            if (cb)
                cb();
        });
    };
    var nodemonInstance = null;
    compile(function () {
        var bundlePath = pathHelper_1.default.serverRelative(config_1.default.paths.server.bundle);
        nodemonInstance = nodemon({ script: bundlePath, flags: [], nodeArgs: ["--debug=" + config_1.default.server.dev.debugPort] })
            .on('quit', process.exit);
        process.on('uncaughtException', function (err) {
            console.log(err);
            nodemonInstance.emit('quit');
        });
    });
    var compileRequested = false;
    var onWatch = function () {
        if (compileRequested)
            return;
        compileRequested = true;
        _.delay(function () {
            compile(function () {
                compileRequested = false;
                nodemonInstance.emit('restart');
            });
        }, 200);
    };
    watcher.on('ready', function () {
        watcher
            .on('add', onWatch)
            .on('addDir', onWatch)
            .on('change', onWatch)
            .on('unlink', onWatch)
            .on('unlinkDir', onWatch);
    });
}
dev();
//# sourceMappingURL=dev.js.map